@@ -43,12 +43,7 @@ def remove_include(k1: NewBar, k2: NewBar, k3: RawBar):
         else:
             raise ValueError
 
-        if k3.open > k3.close:
-            open_ = high
-            close = low
-        else:
-            open_ = low
-            close = high
+        open_, close = (high, low) if k3.open > k3.close else (low, high)
         vol = k2.vol + k3.vol
         # 这里有一个隐藏Bug，len(k2.elements) 在一些及其特殊的场景下会有超大的数量，具体问题还没找到；
         # 临时解决方案是直接限定len(k2.elements)<=100
@@ -190,6 +185,8 @@ class CZSC:
         self.freq = bars[0].freq
         self.get_signals = get_signals
         self.signals = None
+        # cache 是信号计算过程的缓存容器，需要信号计算函数自行维护
+        self.cache = OrderedDict()
 
         for bar in bars:
             self.update(bar)
@@ -256,14 +253,9 @@ class CZSC:
         else:
             # 当前 bar 是上一根 bar 的时间延伸
             self.bars_raw[-1] = bar
-            if len(self.bars_ubi) >= 3:
-                edt = self.bars_ubi[-2].dt
-                self.bars_ubi = [x for x in self.bars_ubi if x.dt <= edt]
-                last_bars = [x for x in self.bars_raw[-50:] if x.dt > edt]
-            else:
-                last_bars = self.bars_ubi[-1].elements
-                last_bars[-1] = bar
-                self.bars_ubi.pop(-1)
+            last_bars = self.bars_ubi.pop(-1).raw_bars
+            assert bar.dt == last_bars[-1].dt
+            last_bars[-1] = bar
 
         # 去除包含关系
         bars_ubi = self.bars_ubi
@@ -321,6 +313,29 @@ class CZSC:
                           title="{}-{}".format(self.symbol, self.freq.value))
         return chart
 
+    def to_plotly(self):
+        """使用 plotly 绘制K线分析图"""
+        import pandas as pd
+        from czsc.utils.plotly_plot import KlineChart
+
+        bi_list = self.bi_list
+        df = pd.DataFrame(self.bars_raw)
+        kline = KlineChart(n_rows=3, title="{}-{}".format(self.symbol, self.freq.value))
+        kline.add_kline(df, name="")
+        kline.add_sma(df, ma_seq=(5, 10, 21), row=1, visible=True, line_width=1.2)
+        kline.add_sma(df, ma_seq=(34, 55, 89, 144), row=1, visible=False, line_width=1.2)
+        kline.add_vol(df, row=2)
+        kline.add_macd(df, row=3)
+
+        if len(bi_list) > 0:
+            bi = pd.DataFrame([{'dt': x.fx_a.dt, "bi": x.fx_a.fx, "text": x.fx_a.mark.value} for x in bi_list] +
+                              [{'dt': bi_list[-1].fx_b.dt, "bi": bi_list[-1].fx_b.fx,
+                                "text": bi_list[-1].fx_b.mark.value[0]}])
+            fx = pd.DataFrame([{'dt': x.dt, "fx": x.fx} for x in self.fx_list])
+            kline.add_scatter_indicator(fx['dt'], fx['fx'], name="分型", row=1, line_width=2)
+            kline.add_scatter_indicator(bi['dt'], bi['bi'], name="笔", text=bi['text'], row=1, line_width=2)
+        return kline.fig
+
     def open_in_browser(self, width: str = "1400px", height: str = '580px'):
         """直接在浏览器中打开分析结果
 
@@ -349,17 +364,16 @@ class CZSC:
 
     @property
     def finished_bis(self) -> List[BI]:
-        """返回当下基本确认完成的笔列表"""
+        """已完成的笔"""
         if not self.bi_list:
             return []
-        else:
-            if self.last_bi_extend:
-                return self.bi_list[:-1]
+        if len(self.bars_ubi) < 5:
+            return self.bi_list[:-1]
         return self.bi_list
 
     @property
     def ubi_fxs(self) -> List[FX]:
-        """返回当下基本确认完成的笔列表"""
+        """bars_ubi 中的分型"""
         if not self.bars_ubi:
             return []
         else:
@@ -367,7 +381,7 @@ class CZSC:
 
     @property
     def fx_list(self) -> List[FX]:
-        """返回当下基本确认完成的笔列表"""
+        """分型列表，包括 bars_ubi 中的分型"""
         fxs = []
         for bi_ in self.bi_list:
             fxs.extend(bi_.fxs[1:])
