@@ -8,13 +8,18 @@ Tushare 数据缓存，这是用pickle缓存数据，是临时性的缓存。
 单次缓存，多次使用，但是不做增量更新，适用于研究场景。
 数据缓存是一种用空间换时间的方法，需要有较大磁盘空间，跑全市场至少需要50GB以上。
 """
+import time
 import os.path
 import shutil
 import pandas as pd
+from tqdm import tqdm
+from typing import List
 from deprecated import deprecated
-from czsc.data.ts import *
+from datetime import timedelta, datetime
 from czsc import envs
+from czsc.enum import Freq
 from czsc.utils import io
+from czsc.data.ts import ts, pro, format_kline, dt_fmt, date_fmt
 
 
 def update_bars_return(kline: pd.DataFrame, bar_numbers=None):
@@ -167,6 +172,7 @@ class TsDataCache:
                 print(f"ths_index: read cache {file_cache}")
         else:
             df = pro.ths_member(ts_code=ts_code, fields="ts_code,code,name,weight,in_date,out_date,is_new")
+            df = df.reset_index(drop=True, inplace=False)
             df.to_feather(file_cache)
         return df
 
