@@ -1,5 +1,4 @@
 # coding: utf-8
-
 from enum import Enum
 
 
@@ -10,31 +9,32 @@ class Operate(Enum):
     HO = "持币"  # Hold Other
 
     # 多头操作
-    LO = "开多"          # Long Open
-    LE = "平多"          # Long Exit
-    LA1 = "第一次加多仓"  # Long Add 1
-    LA2 = "第二次加多仓"  # Long Add 2
-    LR1 = "第一次减多仓"  # Long Reduce 1
-    LR2 = "第二次减多仓"  # Long Reduce 2
+    LO = "开多"  # Long Open
+    LE = "平多"  # Long Exit
 
     # 空头操作
-    SO = "开空"          # Short Open
-    SE = "平空"          # Short Exit
-    SA1 = "第一次加空仓"  # Short Add 1
-    SA2 = "第二次加空仓"  # Short Add 2
-    SR1 = "第一次减空仓"  # Short Reduce 1
-    SR2 = "第二次减空仓"  # Short Reduce 2
+    SO = "开空"  # Short Open
+    SE = "平空"  # Short Exit
+
+    def __str__(self):
+        return self.value
 
 
 class Mark(Enum):
     D = "底分型"
     G = "顶分型"
 
+    def __str__(self):
+        return self.value
+
 
 class Direction(Enum):
     Up = "向上"
     Down = "向下"
 
+    def __str__(self):
+        return self.value
+
 
 class Freq(Enum):
     Tick = "Tick"
@@ -48,3 +48,6 @@ class Freq(Enum):
     M = "月线"
     S = "季线"
     Y = "年线"
+
+    def __str__(self):
+        return self.value
