@@ -11,23 +11,38 @@ from . import bxt
 # ======================================================================================================================
 # 以下是 0.9.1 开始的新标准下实现的信号函数，规范定义：
 # 1. 前缀3个字符区分信号类别
-# 2. 后缀 V221107 之类的标识同一个信号函数的版本
+# 2. 后缀 V221107 之类的标识同一个信号函数的不同版本
 # ======================================================================================================================
 
 from czsc.signals.cxt import (
     cxt_fx_power_V221107,
     cxt_first_buy_V221126,
     cxt_first_sell_V221126,
-    cxt_bi_break_V221126,
-    cxt_sub_b3_V221212,
     cxt_zhong_shu_gong_zhen_V221221,
     cxt_bi_end_V230222,
     cxt_bi_end_V230224,
+    cxt_bi_end_V230104,
+    cxt_bi_end_V230105,
+    cxt_bi_end_V230312,
+    cxt_bi_end_V230320,
+    cxt_bi_end_V230322,
+    cxt_bi_end_V230324,
+    cxt_bi_base_V230228,
+    cxt_third_buy_V230228,
+    cxt_double_zs_V230311,
+    cxt_third_bs_V230318,
+    cxt_third_bs_V230319,
+    cxt_second_bs_V230320,
+    cxt_bi_status_V230101,
+    cxt_bi_status_V230102,
 )
 
 
 from czsc.signals.byi import (
     byi_symmetry_zs_V221107,
+    byi_bi_end_V230106,
+    byi_bi_end_V230107,
+    byi_second_bs_V230324,
 )
 
 from czsc.signals.coo import (
@@ -37,13 +52,14 @@ from czsc.signals.coo import (
 from czsc.signals.vol import (
     vol_single_ma_V230214,
     vol_double_ma_V230214,
+    vol_ti_suo_V221216,
+    vol_gao_di_V221218,
 )
 
 from czsc.signals.bar import (
     bar_end_V221111,
     bar_operate_span_V221111,
-    bar_zdt_V221110,
-    bar_zdt_V221111,
+    bar_zdt_V230331,
     bar_cross_ps_V221112,
     bar_section_momentum_V221112,
     bar_vol_grow_V221112,
@@ -57,12 +73,16 @@ from czsc.signals.bar import (
     bar_amount_acc_V230214,
     bar_big_solid_V230215,
     bar_vol_bs1_V230224,
+    bar_reversal_V230227,
+    bar_bpm_V230227,
+    bar_time_V230327,
+    bar_weekday_V230328,
+    bar_r_breaker_V230326,
 )
 
 from czsc.signals.jcc import (
     jcc_san_xing_xian_V221023,
     jcc_ten_mo_V221028,
-    jcc_bai_san_bin_V221030,
     jcc_san_fa_V20221115,
     jcc_san_fa_V20221118,
     jcc_wu_yun_gai_ding_V221101,
@@ -75,7 +95,6 @@ from czsc.signals.jcc import (
     jcc_zhuo_yao_dai_xian_v221113,
     jcc_two_crow_V221108,
     jcc_three_crow_V221108,
-    jcc_three_soldiers_V221030,
     jcc_szx_V221111,
     jcc_ta_xing_V221124,
     jcc_san_szx_V221122,
@@ -101,21 +120,32 @@ from czsc.signals.tas import (
     tas_macd_first_bs_V221201,
     tas_macd_first_bs_V221216,
     tas_macd_second_bs_V221201,
+    tas_macd_bs1_V230313,
+    tas_macd_bs1_V230312,
+    tas_macd_base_V230320,
 
     tas_ma_base_V221101,
     tas_ma_base_V221203,
+    tas_ma_base_V230313,
     tas_ma_round_V221206,
     tas_double_ma_V221203,
 
     tas_boll_power_V221112,
     tas_boll_bc_V221118,
+    tas_boll_vt_V230212,
+    tas_boll_cc_V230312,
 
     tas_kdj_base_V221101,
     tas_kdj_evc_V221201,
 
-    tas_double_rsi_V221203,
+    # tas_double_rsi_V221203,
+    tas_rsi_base_V230227,
 
     tas_first_bs_V230217,
+    tas_second_bs_V230228,
+    tas_second_bs_V230303,
+
+    tas_hlma_V230301,
 )
 
 
