@@ -22,21 +22,23 @@ from czsc.analyze import CZSC
 from czsc.objects import Position, RawBar, Signal
 from czsc.utils.bar_generator import BarGenerator
 from czsc.utils.cache import home_path
+from czsc.utils import sorted_freqs, import_by_name
+from czsc.traders.sig_parse import get_signals_freqs
 
 
 class CzscSignals:
     """缠中说禅技术分析理论之多级别信号计算"""
 
-    def __init__(self, bg: BarGenerator = None, get_signals: Callable = None):
+    def __init__(self, bg: BarGenerator = None,  **kwargs):
         """
 
         :param bg: K线合成器
-        :param get_signals: 信号计算函数
         """
         self.name = "CzscSignals"
-        self.get_signals: Callable = get_signals
         # cache 是信号计算过程的缓存容器，需要信号计算函数自行维护
         self.cache = OrderedDict()
+        self.kwargs = kwargs
+        self.signals_config = kwargs.get("signals_config", [])
 
         if bg:
             self.bg = bg
@@ -48,11 +50,9 @@ class CzscSignals:
 
             last_bar = self.kas[self.base_freq].bars_raw[-1]
             self.end_dt, self.bid, self.latest_price = last_bar.dt, last_bar.id, last_bar.close
-            if self.get_signals:
-                self.s = self.get_signals(self)
-                self.s.update(last_bar.__dict__)
-            else:
-                self.s = OrderedDict()
+            self.s = OrderedDict(last_bar.__dict__)
+            self.s.update(self.get_signals_by_conf())
+
         else:
             self.bg = None
             self.symbol = None
@@ -65,6 +65,34 @@ class CzscSignals:
     def __repr__(self):
         return "<{} for {}>".format(self.name, self.symbol)
 
+    def get_signals_by_conf(self):
+        """通过信号参数配置获取信号
+
+        信号参数配置，格式如下：
+            signals_config = [
+                {'name': 'czsc.signals.tas_ma_base_V221101', 'freq': '日线', 'di': 1, 'ma_type': 'SMA', 'timeperiod': 5},
+                {'name': 'czsc.signals.tas_ma_base_V221101', 'freq': '日线', 'di': 5, 'ma_type': 'SMA', 'timeperiod': 5},
+                {'name': 'czsc.signals.tas_double_ma_V221203', 'freq': '日线', 'di': 1, 'ma_seq': (5, 20), 'th': 100},
+                {'name': 'czsc.signals.tas_double_ma_V221203', 'freq': '日线', 'di': 5, 'ma_seq': (5, 20), 'th': 100},
+            ]
+        :return: 信号字典
+        """
+        s = OrderedDict()
+        if not self.signals_config:
+            return s
+
+        for param in self.signals_config:
+            param = dict(param)
+            sig_name = param.pop('name')
+            sig_func = import_by_name(sig_name) if isinstance(sig_name, str) else sig_name
+
+            freq = param.pop('freq', None)
+            if freq in self.kas:    # 如果指定了 freq，那么就使用 CZSC 对象作为输入
+                s.update(sig_func(self.kas[freq], **param))
+            else:                   # 否则使用 CAT 作为输入
+                s.update(sig_func(self, **param))
+        return s
+
     def take_snapshot(self, file_html=None, width: str = "1400px", height: str = "580px"):
         """获取快照
 
@@ -119,24 +147,54 @@ class CzscSignals:
         self.symbol = bar.symbol
         last_bar = self.kas[self.base_freq].bars_raw[-1]
         self.end_dt, self.bid, self.latest_price = last_bar.dt, last_bar.id, last_bar.close
-
-        if self.get_signals:
-            self.s = self.get_signals(self)
-            self.s.update(last_bar.__dict__)
+        self.s = OrderedDict(last_bar.__dict__)
+        self.s.update(self.get_signals_by_conf())
 
 
-def generate_czsc_signals(bars: List[RawBar], get_signals: Callable, freqs: List[AnyStr],
+@deprecated(version="0.9.16", reason="请使用 CzscSignals 类")
+def get_signals_by_conf(cat: CzscSignals, conf):
+    """通过信号参数配置获取信号
+
+    :param cat:
+    :param conf: 信号参数配置，格式如下：
+        conf = [
+            {'name': 'czsc.signals.tas_ma_base_V221101', 'freq': '日线', 'di': 1, 'ma_type': 'SMA', 'timeperiod': 5},
+            {'name': 'czsc.signals.tas_ma_base_V221101', 'freq': '日线', 'di': 5, 'ma_type': 'SMA', 'timeperiod': 5},
+            {'name': 'czsc.signals.tas_double_ma_V221203', 'freq': '日线', 'di': 1, 'ma_seq': (5, 20), 'th': 100},
+            {'name': 'czsc.signals.tas_double_ma_V221203', 'freq': '日线', 'di': 5, 'ma_seq': (5, 20), 'th': 100},
+        ]
+    :return: 信号字典
+    """
+    s = OrderedDict({"symbol": cat.symbol, "dt": cat.end_dt, "close": cat.latest_price})
+    for param in conf:
+        param = dict(param)
+        sig_func = import_by_name(param.pop('name'))
+        freq = param.pop('freq', None)
+        if freq in cat.kas:  # 如果指定了 freq，那么就使用 CZSC 对象作为输入
+            s.update(sig_func(cat.kas[freq], **param))
+        else:                # 否则使用 CAT 作为输入
+            s.update(sig_func(cat, **param))
+    return s
+
+
+def generate_czsc_signals(bars: List[RawBar], signals_config: List[dict],
                           sdt: Union[AnyStr, datetime] = "20170101", init_n: int = 500, df=False, **kwargs):
     """使用 CzscSignals 生成信号
 
     :param bars: 基础周期 K 线序列
-    :param get_signals: 信号计算函数
-    :param freqs: K 线周期序列，不需要填写基础周期
+    :param signals_config: 信号函数配置，格式如下：
+        signals_config = [
+            {'name': 'czsc.signals.tas_ma_base_V221101', 'freq': '日线', 'di': 1, 'ma_type': 'SMA', 'timeperiod': 5},
+            {'name': 'czsc.signals.tas_ma_base_V221101', 'freq': '日线', 'di': 5, 'ma_type': 'SMA', 'timeperiod': 5},
+            {'name': 'czsc.signals.tas_double_ma_V221203', 'freq': '日线', 'di': 1, 'ma_seq': (5, 20), 'th': 100},
+            {'name': 'czsc.signals.tas_double_ma_V221203', 'freq': '日线', 'di': 5, 'ma_seq': (5, 20), 'th': 100},
+        ]
     :param sdt: 信号计算开始时间
     :param init_n: 用于 BarGenerator 初始化的基础周期K线数量
     :param df: 是否返回 df 格式的信号计算结果，默认 False
     :return: 信号计算结果
     """
+    freqs = get_signals_freqs(signals_config)
     freqs = [freq for freq in freqs if freq != bars[0].freq.value]
     sdt = pd.to_datetime(sdt)
     bars_left = [x for x in bars if x.dt < sdt]
@@ -159,7 +217,8 @@ def generate_czsc_signals(bars: List[RawBar], get_signals: Callable, freqs: List
         bg.update(bar)
 
     _sigs = []
-    cs = CzscSignals(bg, get_signals)
+    cs = CzscSignals(bg, signals_config=signals_config, **kwargs)
+    cs.cache.update({'gsc_kwargs': kwargs})
     for bar in tqdm(bars_right, desc=f'generate signals of {bg.symbol}'):
         cs.update_signals(bar)
         _sigs.append(dict(cs.s))
@@ -170,13 +229,13 @@ def generate_czsc_signals(bars: List[RawBar], get_signals: Callable, freqs: List
         return _sigs
 
 
-def check_signals_acc(bars: List[RawBar], get_signals: Callable, delta_days: int = 5, **kwargs) -> None:
+def check_signals_acc(bars: List[RawBar], signals_config: List[dict], delta_days: int = 5, **kwargs) -> None:
     """人工验证形态信号识别的准确性的辅助工具：
 
     输入基础周期K线和想要验证的信号，输出信号识别结果的快照
 
     :param bars: 原始K线
-    :param get_signals: 需要验证的信号列表
+    :param signals_config: 需要验证的信号列表
     :param delta_days: 两次相同信号之间的间隔天数
     :return: None
     """
@@ -185,13 +244,7 @@ def check_signals_acc(bars: List[RawBar], get_signals: Callable, delta_days: int
     if len(bars) < 600:
         return
 
-    if not kwargs.get('freqs', None):
-        sorted_freqs = ['1分钟', '5分钟', '15分钟', '30分钟', '60分钟', '日线', '周线', '月线', '季线', '年线']
-        freqs = sorted_freqs[sorted_freqs.index(base_freq) + 1:]
-    else:
-        freqs = kwargs['freqs']
-
-    df = generate_czsc_signals(bars, get_signals, freqs, df=True, **kwargs)
+    df = generate_czsc_signals(bars, signals_config=signals_config, df=True, **kwargs)
     s_cols = [x for x in df.columns if len(x.split("_")) == 3]
     signals = []
     for col in s_cols:
@@ -203,11 +256,12 @@ def check_signals_acc(bars: List[RawBar], get_signals: Callable, delta_days: int
 
     bars_left = bars[:500]
     bars_right = bars[500:]
+    freqs = get_signals_freqs(signals_config)
     bg = BarGenerator(base_freq=base_freq, freqs=freqs, max_count=5000)
     for bar in bars_left:
         bg.update(bar)
 
-    ct = CzscSignals(bg, get_signals)
+    ct = CzscSignals(bg, signals_config=signals_config, **kwargs)
     last_dt = {signal.key: ct.end_dt for signal in signals}
 
     for bar in tqdm(bars_right, desc=f'signals of {bg.symbol}'):
@@ -224,11 +278,31 @@ def check_signals_acc(bars: List[RawBar], get_signals: Callable, delta_days: int
                 last_dt[signal.key] = bar.dt
 
 
+def get_unique_signals(bars: List[RawBar], signals_config: List[dict], **kwargs):
+    """获取信号函数中定义的所有信号列表
+
+    :param bars: 基础K线数据
+    :param signals_config: 信号函数配置
+    :param kwargs:
+    :return:
+    """
+    base_freq = str(bars[-1].freq.value)
+    assert bars[2].dt > bars[1].dt > bars[0].dt and bars[2].id > bars[1].id, "bars 中的K线元素必须按时间升序"
+    if len(bars) < 600:
+        return []
+
+    df = generate_czsc_signals(bars, signals_config=signals_config, df=True, **kwargs)
+    _res = []
+    for col in [x for x in df.columns if len(x.split("_")) == 3]:
+        _res.extend([f"{col}_{v}" for v in df[col].unique() if "其他" not in v])
+    return _res
+
+
 class CzscTrader(CzscSignals):
     """缠中说禅技术分析理论之多级别联立交易决策类（支持多策略独立执行）"""
 
-    def __init__(self, bg: BarGenerator = None, get_signals: Callable = None,
-                 positions: List[Position] = None, ensemble_method: Union[AnyStr, Callable] = "mean"):
+    def __init__(self, bg: BarGenerator = None, positions: List[Position] = None,
+                 ensemble_method: Union[AnyStr, Callable] = "mean", **kwargs):
         """
 
         :param bg: bar generator 对象
@@ -242,9 +316,16 @@ class CzscTrader(CzscSignals):
 
             对于传入回调函数的情况，输入是 self.positions
         """
-        super().__init__(bg, get_signals=get_signals)
         self.positions = positions
+        if self.positions:
+            _pos_names = [x.name for x in self.positions]
+            assert len(_pos_names) == len(set(_pos_names)), "仓位策略名称不能重复"
         self.__ensemble_method = ensemble_method
+        self.name = "CzscTrader"
+        super().__init__(bg, **kwargs)
+
+    def __repr__(self):
+        return "<{} for {}>".format(self.name, self.symbol)
 
     def update(self, bar: RawBar) -> None:
         """输入基础周期已完成K线，更新信号，更新仓位
@@ -338,7 +419,7 @@ class CzscTrader(CzscSignals):
         for freq in self.freqs:
             ka: CZSC = self.kas[freq]
             bs = None
-            if freq == self.base_freq:
+            if freq == self.base_freq and self.positions:
                 # 在基础周期K线上加入最近的操作记录
                 bs = []
                 for pos in self.positions:
