@@ -8,7 +8,7 @@ create_dt: 2021/7/16 11:51
 import os
 import shutil
 
-home_path = os.path.join(os.path.expanduser("~"), '.czsc')
+home_path = os.environ.get("CZSC_HOME", os.path.join(os.path.expanduser("~"), '.czsc'))
 os.makedirs(home_path, exist_ok=True)
 
 
